﻿IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[FNS_InternalLinks_GetLinkPerProduct]') AND type in (N'P', N'PC'))
DROP PROCEDURE [FNS_InternalLinks_GetLinkPerProduct]
GO
/****** Object:  StoredProcedure [dbo].[FNS_InternalLinks_GetLinkPerProduct]    Script Date: 07/19/2014 11:26:55 ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE PROCEDURE [FNS_InternalLinks_GetLinkPerProduct]
(
	@productId int ,	--Product identifier
	@languageId		int,	--Language identifier
	@storeId			int, -- Store identifier
	@createnew			bit, --create new rows
	@AllowedCustomerRoleIds	nvarchar(MAX),	--a list of customer role IDs (comma-separated list) for which a product should be shown (if a subjet to ACL)
	@IgnoreAcl			bit,  --A value indicating whether to Ignore Acl
	@IgnoreStoreLimitations bit --A value indicating whether to Ignore Store Limitations
)
AS
BEGIN
SET NOCOUNT ON

CREATE TABLE #tmpInternalLinks_Product
(
	Id int not null,
	ParentCategoryId int not null,
	IsHide bit not null
)

insert #tmpInternalLinks_Product (Id,ParentCategoryId,IsHide)
select Id,ParentCategoryId,IsHide
from FNS_InternalLinks_Product L WITH (NOLOCK)
where L.ProductId=@productId and L.LanguageId=@languageId and L.StoreId=@storeId

--filter by customer role IDs (access control list)
SET @AllowedCustomerRoleIds = isnull(@AllowedCustomerRoleIds, '')	
CREATE TABLE #FilteredCustomerRoleIds
(
	CustomerRoleId int not null
)
--must create new rows
if not exists (select * from #tmpInternalLinks_Product) and @createnew=1
begin
	create table #CategoryIdLink (Id int not null)
	--Category
	;with cte_tree (Id,ParentCategoryId) as 
	(
		select C.Id,C.ParentCategoryId
		from Product_Category_Mapping Pc  WITH (NOLOCK),Category C  WITH (NOLOCK)
		where Pc.CategoryId=C.Id 
			and  Pc.ProductId=@productId
			and C.Deleted=0
			and C.Published=1
		union all
		select T.Id,T.ParentCategoryId
		from cte_tree C,Category T WITH (NOLOCK)
		where C.ParentCategoryId=T.Id and T.Deleted=0 and T.Published=1
	)
	
	insert into #CategoryIdLink (Id)			
	select distinct Id
	from cte_tree

	create table #tmpFNS_InternalLinks_Words (CategoryId int,Words nvarchar(max)) 
	insert into #tmpFNS_InternalLinks_Words (CategoryId,Words)
	select W.CategoryId,W.Words
	from FNS_InternalLinks_Words W,#CategoryIdLink C
	where W.CategoryId=C.Id and W.LanguageId=@languageId and W.StoreId=@storeId
	
	drop table #CategoryIdLink 

	--scan #tmpFNS_InternalLinks_Words
		declare @categoryId int,@Words nvarchar(max),@word nvarchar(max),@recno int,@reccount int
		DECLARE View_InternalLinks_Words INSENSITIVE CURSOR
		FOR SELECT CategoryId,Words
		FROM #tmpFNS_InternalLinks_Words
		where ltrim(rtrim(Words))!=''
		FOR READ ONLY

		OPEN View_InternalLinks_Words
		FETCH NEXT FROM View_InternalLinks_Words into @categoryId,@Words
		WHILE @@Fetch_Status=0
		begin
			SET @Words = ltrim(rtrim(isnull(@Words, '')))
			SET @Words=REPLACE(@Words,'.',',');
			
			CREATE TABLE #tmpWords
			(
				Id int identity (1,1),
				word nvarchar(max) not null
			)
				
			INSERT INTO #tmpWords (word)
			SELECT ltrim(rtrim(data)) FROM [nop_splitstring_to_table](@Words, ',')
			where ltrim(rtrim(data))!=''
			
			set @reccount=@@ROWCOUNT
			if @reccount=1
			begin
				select @word=ltrim(rtrim(word)) from #tmpWords
			end
			else
			begin
				create table #tmpWords2 (id int IDENTITY(1,1) NOT NULL, Word nvarchar(max),cnt int)
				insert into #tmpWords2 (Word,cnt)
				select W.Word,SUM(CASE WHEN P.Id IS NULL THEN 0 ELSE 1 END) as cnt
				from #tmpWords W left join FNS_InternalLinks_Product P WITH (NOLOCK) on W.word collate SQL_Latin1_General_CP1_CI_AS=P.Word collate SQL_Latin1_General_CP1_CI_AS
				group by W.word
				order by 2
				select top 1 @word=ltrim(rtrim(word)) from #tmpWords2
				
				--select * from #tmpWords2
				drop table #tmpWords2
			end
			
			drop table #tmpWords
			--select @word

			if len(@word)=1
				set @word=upper(@word)
			else
				set @word=upper(substring(@word,1,1))+substring(@word,2,len(@word)-1)
							
			--@Words = 'Buy in Town1,Buy in Town2, Buy in Town3,Buy in Town4.'
			if @word!=''
			begin
				INSERT INTO FNS_InternalLinks_Product(ParentCategoryId,ProductId,LanguageId,StoreId,Word,IsHide)
				values (@categoryId,@productId,@languageId,@storeId,@word,0)
			end
			FETCH NEXT FROM View_InternalLinks_Words into @categoryId,@Words
		end	
		DEALLOCATE View_InternalLinks_Words
	
	
	drop table #tmpFNS_InternalLinks_Words
	
	insert #tmpInternalLinks_Product (Id,ParentCategoryId,IsHide)
	select Id,ParentCategoryId,IsHide
	from FNS_InternalLinks_Product L WITH (NOLOCK)
	where L.ProductId=@productId and L.LanguageId=@languageId and L.StoreId=@storeId	
end

--select table
	create table #CategoryId (Id int not null, SubjectToAcl bit,LimitedToStores bit)
	insert into #CategoryId (Id,SubjectToAcl,LimitedToStores)
	select Id,SubjectToAcl,LimitedToStores
	from Category C
	where C.Id in (select ParentCategoryId from #tmpInternalLinks_Product where IsHide=0)
		and C.Published=1 and C.Deleted=0

    --ACL Category
	if @IgnoreAcl=0 and exists(select Id from #CategoryId where SubjectToAcl=1)		
	begin
		INSERT INTO #FilteredCustomerRoleIds (CustomerRoleId)
		SELECT CAST(data as int) FROM [nop_splitstring_to_table](@AllowedCustomerRoleIds, ',')	

		delete #CategoryId
		from #CategoryId C
		where C.SubjectToAcl=1 and not EXISTS (
								SELECT 1 FROM #FilteredCustomerRoleIds [fcr]
									WHERE
										[fcr].CustomerRoleId IN (
											SELECT [acl].CustomerRoleId
											FROM [AclRecord] acl  WITH (NOLOCK)
											WHERE [acl].EntityId = C.Id AND [acl].EntityName = 'Category'
										)
								)
	end
	--StoreMapping Category
	if @IgnoreStoreLimitations=0 and  exists(select Id from #CategoryId where LimitedToStores=1)		
	begin
		delete #CategoryId
		from #CategoryId C
		where C.LimitedToStores=1 and not Exists(select Top 1 Id from StoreMapping SM WITH (NOLOCK)
					where sm.EntityName = 'Category' and sm.StoreId=@storeId and sm.EntityId=C.Id)
	end

	select P.* 
	from FNS_InternalLinks_Product P WITH (NOLOCK),FNS_InternalLinks_Words W
	where P.Id in (select Id 
			from #tmpInternalLinks_Product 
			where IsHide=0 and ParentCategoryId in (select Id from #CategoryId))
		  and P.ParentCategoryId=W.CategoryId
		  and W.Published=1 and W.Deleted=0
		  and W.LanguageId=P.LanguageId and W.StoreId=P.StoreId

	drop table #CategoryId

	drop table #tmpInternalLinks_Product
	drop table #FilteredCustomerRoleIds

END
GO
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[FNS_InternalLinks_GetLinkPerCategory]') AND type in (N'P', N'PC'))
DROP PROCEDURE [FNS_InternalLinks_GetLinkPerCategory]
GO
/****** Object:  StoredProcedure [dbo].[FNS_InternalLinks_GetLinkPerCategory]    Script Date: 07/19/2014 11:24:11 ******/
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE PROCEDURE [FNS_InternalLinks_GetLinkPerCategory]
(
	@categoryId int ,	--Category identifier
	@languageId		int,	--Language identifier
	@storeId			int, -- Store identifier
	@createnew			bit, --create new rows
	@AllowedCustomerRoleIds	nvarchar(MAX),	--a list of customer role IDs (comma-separated list) for which a product should be shown (if a subjet to ACL)
	@IgnoreAcl			bit,  --A value indicating whether to Ignore Acl
	@IgnoreStoreLimitations bit --A value indicating whether to Ignore Store Limitations
)
AS
BEGIN
SET NOCOUNT ON

CREATE TABLE #tmpInternalLinks_Category
(
	Id int not null,
	ParentCategoryId int not null,
	IsHide bit not null
)

insert #tmpInternalLinks_Category (Id,ParentCategoryId,IsHide)
select Id,ParentCategoryId,IsHide
from FNS_InternalLinks_Category L WITH (NOLOCK)
where L.CategoryId=@categoryId and L.LanguageId=@languageId and L.StoreId=@storeId

--filter by customer role IDs (access control list)
SET @AllowedCustomerRoleIds = isnull(@AllowedCustomerRoleIds, '')	
CREATE TABLE #FilteredCustomerRoleIds
(
	CustomerRoleId int not null
)
--must create new rows
if not exists (select * from #tmpInternalLinks_Category) and @createnew=1
begin
	create table #CategoryIdLink (Id int not null)

	--Category
	;with cte_tree (Id,ParentCategoryId) as 
	(
		select C.Id,C.ParentCategoryId
		from Category C  WITH (NOLOCK)
		where C.Id=@categoryId
			and C.Deleted=0
			and C.Published=1
		union all
		select T.Id,T.ParentCategoryId
		from cte_tree C,Category T WITH (NOLOCK)
		where C.ParentCategoryId=T.Id and T.Deleted=0 and T.Published=1
	)
	
	insert into #CategoryIdLink (Id)			
	select Id
	from cte_tree

	--select * from #CategoryIdLink
	create table #tmpFNS_InternalLinks_Words (CategoryId int,Words nvarchar(max)) 
	insert into #tmpFNS_InternalLinks_Words (CategoryId,Words)
	select W.CategoryId,W.Words
	from FNS_InternalLinks_Words W WITH (NOLOCK, index(IX_FNS_InternalLinks_Words_CategoryId)),#CategoryIdLink C
	where W.CategoryId=C.Id and W.LanguageId=@languageId and W.StoreId=@storeId
		and W.CategoryId!=@categoryId	
	drop table #CategoryIdLink 

	--scan #tmpFNS_InternalLinks_Words
				
		declare @parentcategoryId int,@Words nvarchar(max),@word nvarchar(max),@recno int,@reccount int
		DECLARE View_InternalLinks_Words INSENSITIVE CURSOR
		FOR SELECT CategoryId,Words
		FROM #tmpFNS_InternalLinks_Words
		where ltrim(rtrim(Words))!=''
		FOR READ ONLY

		OPEN View_InternalLinks_Words
		FETCH NEXT FROM View_InternalLinks_Words into @parentcategoryId,@Words
		WHILE @@Fetch_Status=0
		begin
			SET @Words = ltrim(rtrim(isnull(@Words, '')))
			SET @Words=REPLACE(@Words,'.',',');
			
			CREATE TABLE #tmpWords
			(
				Id int identity (1,1),
				word nvarchar(max) not null
			)
				
			INSERT INTO #tmpWords (word)
			SELECT ltrim(rtrim(data)) FROM [nop_splitstring_to_table](@Words, ',')
			where ltrim(rtrim(data))!=''
			
			set @reccount=@@ROWCOUNT
			if @reccount=1
			begin
				select @word=ltrim(rtrim(word)) from #tmpWords
			end
			else
			begin
				create table #tmpWords2 (id int IDENTITY(1,1) NOT NULL, Word nvarchar(max),cnt int)
				insert into #tmpWords2 (Word,cnt)
				select W.Word,SUM(CASE WHEN P.Id IS NULL THEN 0 ELSE 1 END) as cnt
				from #tmpWords W left join FNS_InternalLinks_Category P WITH (NOLOCK) on W.word collate SQL_Latin1_General_CP1_CI_AS=P.Word collate SQL_Latin1_General_CP1_CI_AS
				group by W.word
				order by 2
				select top 1 @word=ltrim(rtrim(word)) from #tmpWords2
				
				--select * from #tmpWords2
				drop table #tmpWords2
			end
			
			drop table #tmpWords
						
			--select @word
			if len(@word)=1
				set @word=upper(@word)
			else
				set @word=upper(substring(@word,1,1))+substring(@word,2,len(@word)-1)
				
			--@Words = 'Buy in Town1,Buy in Town2, Buy in Town3,Buy in Town4.'
			if @word!=''
			begin
				INSERT INTO FNS_InternalLinks_Category(ParentCategoryId,CategoryId,LanguageId,StoreId,Word,IsHide)
				values (@parentcategoryId,@categoryId,@languageId,@storeId,@word,0)
			end
			FETCH NEXT FROM View_InternalLinks_Words into @parentcategoryId,@Words
		end	
		DEALLOCATE View_InternalLinks_Words
	
	
	drop table #tmpFNS_InternalLinks_Words
	
	insert #tmpInternalLinks_Category (Id,ParentCategoryId,IsHide)
	select Id,ParentCategoryId,IsHide
	from FNS_InternalLinks_Category L WITH (NOLOCK, index(IX_FNS_InternalLinks_Category_CategoryId))
	where L.CategoryId=@categoryId and L.LanguageId=@languageId and L.StoreId=@storeId	
end

--select table
	create table #CategoryId (Id int not null, SubjectToAcl bit,LimitedToStores bit)
	insert into #CategoryId (Id,SubjectToAcl,LimitedToStores)
	select Id,SubjectToAcl,LimitedToStores
	from Category C
	where C.Id in (select ParentCategoryId from #tmpInternalLinks_Category where IsHide=0)
		and C.Published=1 and C.Deleted=0

    --ACL Category
	if @IgnoreAcl=0 and exists(select Id from #CategoryId where SubjectToAcl=1)		
	begin
		INSERT INTO #FilteredCustomerRoleIds (CustomerRoleId)
		SELECT CAST(data as int) FROM [nop_splitstring_to_table](@AllowedCustomerRoleIds, ',')	

		delete #CategoryId
		from #CategoryId C
		where C.SubjectToAcl=1 and not EXISTS (
								SELECT 1 FROM #FilteredCustomerRoleIds [fcr]
									WHERE
										[fcr].CustomerRoleId IN (
											SELECT [acl].CustomerRoleId
											FROM [AclRecord] acl  WITH (NOLOCK)
											WHERE [acl].EntityId = C.Id AND [acl].EntityName = 'Category'
										)
								)
	end
	--StoreMapping Category
	if @IgnoreStoreLimitations=0 and exists(select Id from #CategoryId where LimitedToStores=1)		
	begin
		delete #CategoryId
		from #CategoryId C
		where C.LimitedToStores=1 and not Exists(select Top 1 Id from StoreMapping SM WITH (NOLOCK)
					where sm.EntityName = 'Category' and sm.StoreId=@storeId and sm.EntityId=C.Id)
	end

	select P.* 
	from FNS_InternalLinks_Category P WITH (NOLOCK),FNS_InternalLinks_Words W WITH (NOLOCK, index(IX_FNS_InternalLinks_Words_CategoryId))
	where P.Id in (select Id 
			from #tmpInternalLinks_Category
			where IsHide=0 and ParentCategoryId in (select Id from #CategoryId))
		  and P.ParentCategoryId=W.CategoryId
		  and W.Published=1 and W.Deleted=0
		  and W.LanguageId=P.LanguageId and W.StoreId=P.StoreId

	drop table #CategoryId

	drop table #tmpInternalLinks_Category
	drop table #FilteredCustomerRoleIds

END
GO

/*
if exists(
	select COUNT(*) 
	from FNS_InternalLinks_Product WITH (NOLOCK)
	group by ParentCategoryId,ProductId, LanguageId, StoreId
	having COUNT(*)>1)
begin
	delete from FNS_InternalLinks_Product
	where Id not in (
		SELECT TOP 1 WITH TIES Id
		FROM FNS_InternalLinks_Product WITH (NOLOCK)
		ORDER BY ROW_NUMBER() OVER(PARTITION BY ParentCategoryId,ProductId, LanguageId, StoreId ORDER BY Id)
	)
end
*/